;; -*- Mode: Emacs-Lisp ; Coding: utf-8 -*-
;; 別冊ムックEmacs特集用 init.el

;; おまじない
(require 'cl)

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; 第2章 最初に知っておきたい基礎知識                     ;;
;; インストール、画面設定、基本操作                       ;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;; P175
;;; リージョンをインデント
;; バッファ全体をインデント
(define-key (kbd "C-<f8>") (kbd "C-x h TAB"))

;; 上記の別の方法
;; (defun indent-whole-buffer ()
;;   (interactive)
;;   (indent-region (point-min) (point-max)))
;; (define-key global-map (kbd "C-<f8>") 'indent-whole-buffer)

;;; P177
;;; リスト1●ファイル名とシバンでメジャーモードを切り替える
;; シバンに perl があれば cperl-mode
(add-to-list 'interpreter-mode-alist '("perl" . cperl-mode))
;; .pl .pm .cgi .fcig .t で終わるファイルは cperl-mode
(add-to-list 'auto-mode-alist '("\\.\\(p[lm]\\|f?cgi\\|t\\)$" . cperl-mode))

;;; リスト2●特定のメジャーモードのときだけマイナーモードを有効化する
;; lisp-mode-hooksを定義
(defun lisp-mode-hooks ()
  "lisp-mode-hooks"
  (require 'eldoc)
  (setq eldoc-idle-delay 0.2)
  (setq eldoc-echo-area-use-multiline-p t)
  (turn-on-eldoc-mode))

;; それぞれのフック変数に関数を追加する
(add-hook 'emacs-lisp-mode-hook 'lisp-mode-hooks)
(add-hook 'lisp-interaction-mode-hook 'lisp-mode-hooks)


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; 第3章 自分好みにカスタマイズするためのポイント         ;;
;; Elisp の基礎、キーバインド、表示設定                   ;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;; P180
;;; インストール環境を整備する
;;; リスト1●init.elを作成
;; load-path を追加する関数を定義
(defun add-to-load-path (&rest paths)
  (let (path)
    (dolist (path paths paths)
      (let ((default-directory (expand-file-name (concat user-emacs-directory path))))
        (add-to-list 'load-path default-directory)
        (if (fboundp 'normal-top-level-add-subdirs-to-load-path)
            (normal-top-level-add-subdirs-to-load-path))))))

;; elispとconfディレクトリをサブディレクトリごとload-pathに追加
(add-to-load-path "elisp" "conf")

;;; conf ディレクトリ内の設定ファイルの読み込み方法の例
;; ~/.emacs.d/conf/init-anything.el というファイルを読み込む場合
;; (load "init-anything")
;; で読み込み可能です。


;;; P180
;;; auto-install.el をインストールする
;;; リスト2●auto-installの設定例
;; (install-elisp "http://www.emacswiki.org/emacs/download/auto-install.el")
(when (require 'auto-install nil t)
  (setq auto-install-directory "~/.emacs.d/elisp/")
  (auto-install-update-emacswiki-package-name t)
  (auto-install-compatibility-setup))


;;; P181
;;; auto-install を利用してみる
;;; リスト3●拡張を今後も利用するように設定
;; redo+: Emacsにredoコマンドを与える
;; (install-elisp "http://www.emacswiki.org/emacs/download/redo+.el")
(when (require 'redo+ nil t)
  (define-key global-map (kbd "C-'") 'redo)) ; C-' に redo を割り当てる


;;; P182
;;; キーバインドの割り当て方法
;;; リスト4●キーバインド割り当ての例
;; "C-m" に newline-and-indent を割り当てる。初期値は newline
(define-key global-map (kbd "C-m") 'newline-and-indent)

;; "M-k" でカレントバッファを閉じる。初期値は kill-sentence
(define-key global-map (kbd "M-k") 'kill-this-buffer)

;; "C-t" でウィンドウを切り替える。初期値は transpose-chars
(define-key global-map (kbd "C-t") 'other-window)

;; Mac の command + → でウィンドウを左右に分割
(define-key global-map (kbd "s-<right>") 'split-window-horizontally)
;; Mac の Command + ↓ でウィンドウを上下に分割
(define-key global-map (kbd "s-<down>") 'split-window-vertically)
;; Mac の Command + w で現在のウィンドを削除
(define-key global-map (kbd "s-w") 'delete-window)

;; cperl-mode で "M-p" で cperl-perldoc
(define-key cperl-mode-map (kbd "M-p") 'cperl-perldoc)


;;; P183
;;; パスを追加する
;; exec-pathリストにパスを追加する
(add-to-list 'exec-path "/opt/local/bin")
(add-to-list 'exec-path "/opt/local/sbin")
(add-to-list 'exec-path "/usr/local/bin")
(add-to-list 'exec-path "/usr/local/sbin")
(add-to-list 'exec-path "~/bin")

;; 環境変数 PATH に exec-path を追加する。
(setenv "PATH" (mapconcat 'identity exec-path ":"))


;;; P183
;;; 文字コードと言語を設定する
;; 言語設定
(set-language-environment "Japanese")
(prefer-coding-system 'utf-8-unix)
(set-default-coding-systems 'utf-8-unix)
;; MacOSX
(require 'ucs-normalize)
(setq file-name-coding-system 'utf-8-hfs)
(setq locale-coding-system 'utf-8-hfs))
;; Windows
;; (setq file-name-coding-system 'sjis)
;; (setq locale-coding-system 'utf-8)
;; Linux, etc
;; (setq file-name-coding-system 'utf-8)
;; (setq locale-coding-system 'utf-8)


;;; P183
;;; 不要なものを非表示にする
;; スタートアップメッセージを非表示
(setq inhibit-startup-screen t)
(when window-system
  ;; tool-bar を非表示
  (tool-bar-mode 0)
  ;; scroll-bar を非表示
  (scroll-bar-mode 0))
;; カーソルを点滅
(blink-cursor-mode t)
;; menu-bar を非表示
(menu-bar-mode 0)


;;; P183
;;; 情報を表示する、目立たせる
;; メニューバーにファイルのフルパスを表示
(setq frame-title-format
      (format "%%f - Emacs@%s" (system-name)))
;; paren: 対応する括弧を光らせる
(setq show-paren-delay 0)
(show-paren-mode t)
(setq show-paren-style 'expression)                    ; カッコ内の色も変更
(set-face-background 'show-paren-match-face nil)       ; カッコ内のフェイス
(set-face-underline-p 'show-paren-match-face "yellow") ; カッコ内のフェイス


;;; P184
;;; インデントの設定
(setq js-indent-level 4)
(setq cperl-indent-level 4)


;;; P183
;;; カラーテーマを利用する
;; color-theme
(when window-system
  (require 'color-theme nil t)
  (color-theme-initialize)
  (color-theme-hober))

;;; P184
;;; フェイスで文字の表示を変える
;; リージョンの背景色を変更
(set-face-background 'region "darkgreen")


;;; P184
;;; 行のハイライト
;; hl-line-mode
(defface my-hl-line-face
  ;;  背景が dark ならば 背景を黒に.
  '((((class color) (background dark))
     (:background "NavyBlue" t))
    ;; 背景が light ならば背景色を緑に
    (((class color) (background light))
     (:background "LightGoldenrodYellow" t))
    (t (:bold t)))
  "hl-line's my face")
(setq hl-line-face 'my-hl-line-face)
(global-hl-line-mode t)


;;; P185
;;; フォントを設定する
;; フォント名一覧表示 *scratch* バッファで下のS式を評価する
;; (prin1 (font-family-list))
;; font-setting Emacs23 for Mac
(when (eq window-system 'ns)
  ;; 標準のフォントを設定
  (set-face-attribute 'default nil
                      :family "Menlo"
                      ;; 12pt
                      :height 120)
  ;; 日本語全部
  (set-fontset-font nil 'japanese-jisx0208 (font-spec :family "Hiragino_Mincho_ProN"))
  ;; かなとカナだけ上書き
  ;; (set-fontset-font nil '( #x3040 .  #x30ff) (font-spec :family "NfMotoyaCedar"))

  ;; 横幅を1:2に調節
  (setq face-font-rescale-alist
        '((".*Menlo.*" . 1.0)
          (".*Hiragino_Mincho_ProN.*" . 1.2)
          (".*Hiragino_Kaku_Gothic_ProN.*" . 1.2)
          (".*nfmotoyacedar-bold.*" . 1.2)
          (".*nfmotoyacedar-medium.*" . 1.2))))

;; Windows で英数に Consolas、日本語にメイリオを指定する
(when (eq window-system 'w32)
  (set-face-attribute 'default nil
                      :family "Consolas"
                      :height 110)
  (set-fontset-font nil 'japanese-jisx0208 (font-spec :family "Meiryo")))


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; 第4章 導入必須の便利な拡張機能                         ;;
;; 検索、置換、編集履歴、入力補助の強化                   ;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;; P186
;;; color-moccur: 検索結果をリストアップ
;; (install-elisp "http://www.emacswiki.org/emacs/download/color-moccur.el")
;; (install-elisp "http://www.emacswiki.org/emacs/download/moccur-edit.el")
(when (require 'color-moccur nil t)
  ;; グローバルマップにoccur-by-moccurを割り当て
  (define-key global-map (kbd "M-o") 'occur-by-moccur)
  ;; スペース区切りでAND検索
  (setq moccur-split-word t)
  ;; ディレクトリ検索のとき除外するファイル
  (add-to-list 'dmoccur-exclusion-mask "\\.DS_Store")
  (add-to-list 'dmoccur-exclusion-mask "^#.+#$")
  (require 'moccur-edit nil t)
  ;; migemo 利用できる環境であれば migemo を使う
  (when (and (executable-find "cmigemo")
             (require 'migemo nil t))
    (setq moccur-use-migemo t)))


;;; P187
;; grep-edit: grep から直接置換
;; (install-elisp "http://www.emacswiki.org/emacs/download/grep-edit.el")
(require 'grep-edit)


;;; P187
;;; migemo: ローマ字インクリメンタルサーチ
;; (install-elisp "https://raw.github.com/emacs-jp/migemo/master/migemo.el")
(when (and (executable-find "cmigemo")
           (require 'migemo nil t))
  ;; cmigemoを使う
  (setq migemo-command "cmigemo")
  ;; migemoのコマンドラインオプション
  (setq migemo-options '("-q" "--emacs" "-i" "\a"))
  ;; migemo辞書の場所
  (setq migemo-dictionary "/usr/local/share/migemo/utf-8/migemo-dict")
  ;; cmigemoで必須の設定
  (setq migemo-user-dictionary nil)
  (setq migemo-regex-dictionary nil)
  ;; キャッシュの設定
  (setq migemo-use-pattern-alist t)
  (setq migemo-use-frequent-pattern-alist t)
  (setq migemo-pattern-alist-length 1000)
  (setq migemo-coding-system 'utf-8-unix)
  ;; migemoを起動する
  (migemo-init))


;;; P188
;;; undohist: 閉じたバッファも Undo できる
;; (install-elisp "http://cx4a.org/pub/undohist.el")
(when (require 'undohist nil t)
  (undohist-initialize))


;;; P188
;;; undo-tree: Undo の分岐を視覚化する
;; (install-elisp "http://www.dr-qubit.org/undo-tree/undo-tree.el")
(when (require 'undo-tree nil t)
  (global-undo-tree-mode))


;;; P189
;;; point-undo: カーソル位置を Undo
;; (install-elisp "http://www.emacswiki.org/cgi-bin/wiki/download/point-undo.el")
(when (require 'point-undo nil t)
  (define-key global-map (kbd "M-[") 'point-undo)
  (define-key global-map (kbd "M-]") 'point-redo))

;;; P189
;;; wdiree: dired で直接ファイルをリネーム
(require 'wdired)
(define-key dired-mode-map "r" 'wdired-change-to-wdired-mode)


;;; P190
;;; auto-complete-mode: 高機能補完+ポップアップメニュー
(when (require 'auto-complete-config nil t)
  (add-to-list 'ac-dictionary-directories "~/.emacs.d/elisp/ac-dict")
  (define-key ac-mode-map (kbd "M-TAB") 'auto-complete)
  (ac-config-default))


;;; P191
;;; smartchr: サイクルスニペット
;; (install-elisp "https://raw.github.com/imakado/emacs-smartchr/master/smartchr.el")
(when (require 'smartchr nil t)
  (define-key global-map (kbd "=") (smartchr '("=" " = " " == " " === ")))
  (defun css-mode-hooks ()
    (define-key cssm-mode-map (kbd ":") (smartchr '(": " ":"))))
  
  (add-hook 'css-mode-hook 'css-mode-hooks))


;;; P191
;;; Elscreen: GNU Screenライクなウィンドウ管理を実現
(when (require 'elscreen nil t)
  (if window-system
      (define-key elscreen-map (kbd "C-z") 'iconify-or-deiconify-frame)
    (define-key elscreen-map (kbd "C-z") 'suspend-emacs)))


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; 第5章 Emacsの操作性に革命を起こす「Anything」          ;;
;; 候補の表示、絞り込み、実行を統一的に行うフレームワーク ;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;; P193
;;; Anything の基本設定
;;; リスト1●Anythingの設定サンプル
;; (auto-install-batch "anything")
(when (require 'anything nil t)
  (setq
   ;; 候補を表示するまでの時間。デフォルトは0.5
   anything-idle-delay 0.3
   ;; タイプして再描写するまでの時間。デフォルトは0.1
   anything-input-idle-delay 0.2
   ;; 候補の最大表示数。デフォルトは50
   anything-candidate-number-limit 100
   ;; 候補が多いときに体感速度を早くする
   anything-quick-update t
   ;; 候補選択ショートカットをアルファベットに
   anything-enable-shortcuts 'alphabet)

  (when (require 'anything-config nil t)
    ;; root権限でアクションを実行するときのコマンド
    ;; デフォルトは"su"
    (setq anything-su-or-sudo "sudo"))

  (require 'anything-match-plugin nil t)
  (and (equal current-language-environment "Japanese")
       (executable-find "cmigemo")
       (require 'anything-migemo nil t))
  (when (require 'anything-complete nil t)
    ;; M-xによる補完をAnythingで行なう
    ;; (anything-read-string-mode 1)
    ;; lispシンボルの補完候補の再検索時間
    (anything-lisp-complete-symbol-set-timer 150))

  (require 'anything-show-completion nil t)

  (when (require 'auto-install nil t)
    (require 'anything-auto-install nil t))

  (when (require 'descbinds-anything nil t)
    ;; describe-bindingsをAnythingに置き換える
    (descbinds-anything-install))

  (require 'anything-grep nil t)

  ;;; 特に個人的な設定
  ;; Command+f で anything
  (define-key global-map (kbd "s-f") 'anything)
  ;; Command+y で anything-show-kill-ring
  (define-key global-map (kbd "s-y") 'anything-show-kill-ring)

  ;;; P194
  ;;; manやinfoを調べるコマンドを作成してみる
  ;; anything-for-document 用のソースを定義
  (setq anything-for-document-sources
      (list anything-c-source-man-pages
            anything-c-source-info-cl
            anything-c-source-info-pages
            anything-c-source-info-elisp
            anything-c-source-apropos-emacs-commands
            anything-c-source-apropos-emacs-functions
            anything-c-source-apropos-emacs-variables))
  ;; anything-for-document コマンドを作成
  (defun anything-for-document ()
    "Preconfigured `anything' for anything-for-document."
    (interactive)
    (anything anything-for-document-sources (thing-at-point 'symbol) nil nil nil "*anything for document*"))
  ;; Command+d に anything-for-documentを割り当て
  (define-key global-map (kbd "s-d") 'anything-for-document)

  ;;; P196
  ;;; anything-project: プロジェクトからファイルを絞り込み
  ;; (install-elisp "https://raw.github.com/imakado/anything-project/master/anything-project.el")
  (when (require 'anything-project nil t)
    (global-set-key (kbd "C-c C-f") 'anything-project)
    (setq ap:project-files-filters
          (list
           (lambda (files)
             (remove-if 'file-directory-p files)
             (remove-if '(lambda (file) (string-match-p "~$" file)) files)))))

  ;;; P196
  ;;; anything-c-moccur: MoccurのAnythingインターフェイス
  ;; (install-elisp "http://svn.coderepos.org/share/lang/elisp/anything-c-moccur/trunk/anything-c-moccur.el")
  (when (require 'anything-c-moccur nil t)
    (setq
     ;; anything-c-moccur用 `anything-idle-delay'
     anything-c-moccur-anything-idle-delay 0.1
     ;; バッファの情報をハイライトする
     anything-c-moccur-higligt-info-line-flag t
     ;; 現在選択中の候補の位置を他のwindowに表示する
     anything-c-moccur-enable-auto-look-flag t
     ;; 起動時にポイントの位置の単語を初期パターンにする
     anything-c-moccur-enable-initial-pattern t)
    
    (global-set-key (kbd "C-M-o")
                    'anything-c-moccur-occur-by-moccur))
  )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; 第6章 思考を即コード化する開発環境                     ;;
;; 文法・構文チェック、バージョン管理、システム連携…     ;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;; P198
;;; flymake-mode: 動的構文チェッカ
;; 要 flymake-jsl.el
;; http://d.hatena.ne.jp/kazu-yamamoto/20071029/1193651325
;; js-mode用のhookを用意
(when (executable-find "jsl")
  (defun js-mode-hooks ()
    ;; キーマップをセット
    (setq flymake-jsl-mode-map 'js-mode-map)
    ;; flymake-jslを起動するための設定
    (when (require 'flymake-jsl nil t)
      (setq flymake-check-was-interrupted t)
      (flymake-mode t)))
  ;; js-modeの起動時にhookを追加
  (add-hook 'js-mode-hook 'js-mode-hooks))


;;; P199
;;; flyspell-mode: 動的スペルチェッカ
;; スペルチェッカに aspell を使う
;; (when (executable-find "aspell")
;;   (setq-default ispell-program-name "aspell"))
;;; ポップアップでスペル修正候補を表示する
;; 修正したい単語の上にカーソルをもっていき, C-c s を押すことで候補を選択
(defun flyspell-mode-hooks ()
  (defun flyspell-correct-word-popup-el ()
    "Pop up a menu of possible corrections for misspelled word before point."
    (interactive)
    ;; use the correct dictionary
    (flyspell-accept-buffer-local-defs)
    (let ((cursor-location (point))
          (word (flyspell-get-word nil)))
      (if (consp word)
          (let ((start (car (cdr word)))
                (end (car (cdr (cdr word))))
                (word (car word))
                poss ispell-filter)
            ;; now check spelling of word.
            (ispell-send-string "%\n")	;put in verbose mode
            (ispell-send-string (concat "^" word "\n"))
            ;; wait until ispell has processed word
            (while (progn
                     (accept-process-output ispell-process)
                     (not (string= "" (car ispell-filter)))))
            ;; Remove leading empty element
            (setq ispell-filter (cdr ispell-filter))
            ;; ispell process should return something after word is sent.
            ;; Tag word as valid (i.e., skip) otherwise
            (or ispell-filter
                (setq ispell-filter '(*)))
            (if (consp ispell-filter)
                (setq poss (ispell-parse-output (car ispell-filter))))
            (cond
             ((or (eq poss t) (stringp poss))
              ;; don't correct word
              t)
             ((null poss)
              ;; ispell error
              (error "Ispell: error in Ispell process"))
             (t
              ;; The word is incorrect, we have to propose a replacement.
              (flyspell-do-correct (popup-menu* (car (cddr poss)) :scroll-bar t :margin t)
                                   poss word cursor-location start end cursor-location)))
            (ispell-pdict-save t)))))
  ;; C-c s でスペル修正候補をポップアップ
  (define-key flyspell-mode-map (kbd "C-c s") 'flyspell-correct-word-popup-el))

(add-hook 'flyspell-mode-hook 'flyspell-mode-hooks)


;;; P199
;;; Egg: Git フロントエンド
;; (install-elisp "https://raw.github.com/byplayer/egg/master/egg.el")
(when (executable-find "git")
  (require 'egg nil t))


;;; P200
;;; Emacsから本格的にシェルを使う
;; (install-elisp "http://www.emacswiki.org/emacs/download/multi-term.el")
(when (require 'multi-term nil t)
  (setq multi-term-program "/usr/local/bin/zsh"))


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; 番外 おまけ設定                                        ;;
;; 特集に入りきらなかった設定サンプル                     ;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;; Emacsからの質問を y/n で回答する
(fset 'yes-or-no-p 'y-or-n-p)

;; カーソル位置のファイルパスやアドレスを "C-x C-f" で開く
(ffap-bindings)

;; TAB の表示幅。初期値は8
(setq-default tab-width 4)

;; TAB をスペースに
(setq-default indent-tabs-mode nil)

;;; after-save-hook の利用サンプル
;; growl に通知するための関数を定義
(when (executable-find "growlnotify")
  (defun growlnotify-after-save-hook ()
    (shell-command
     (format
      "growlnotify -a \"Emacs.app\" -t \"バッファを保存\" -m \"%s\""
      (buffer-name (current-buffer)))))

  ;; 関数をafter-save-hookに追加する
  (add-hook 'after-save-hook 'growlnotify-after-save-hook))

;;; 便利な拡張
;; bm (VisibleBookmarks)
;; (install-elisp "http://cvs.savannah.gnu.org/viewvc/*checkout*/bm/bm/bm.el")
;; ;; bmに機能を追加する拡張
;; (install-elisp "http://www.emacswiki.org/emacs/download/bm-ext.el")
(when (and window-system
           (require 'bm nil t))
  (require 'bm-ext nil t)
  ;; bmの記録量
  (setq bm-bookmark-context-size 128)
  (setq bm-repository-size nil)
  ;; キーバインドに割り当て
  (define-key global-map (kbd "M-SPC") 'bm-toggle)
  (define-key global-map (kbd "C-;") 'bm-previous))


;;; cua-mode: 矩形編集を強力にする
;; リージョン選択中に C-RET で矩形編集に入る、C-g で終了
(require 'cua-mode nil t)
(cua-mode t)
(setq cua-enable-cua-keys nil)


;;; psvn: Subversion フロントエンド
;; (install-elisp "http://www.xsteve.at/prg/emacs/psvn.el")
(when (executable-find "svn")
  (setq svn-status-verbose nil)
  (autoload 'svn-status "psvn" "Run `svn status'." t))


;;; php-mode
;; (install-elisp "https://raw.github.com/ejmr/php-mode/master/php-mode.el")
(when (require 'php-mode nil t)
  ;; 拡張子 .ctp も php-mode で開く
  (add-to-list 'auto-mode-alist '("\\.ctp$" . php-mode)))

;; emacs-cake
;; https://github.com/k1LoW/emacs-cake/wiki
;; (install-elisp "https://raw.github.com/k1LoW/emacs-historyf/master/historyf.el")
;; (install-elisp "https://raw.github.com/k1LoW/emacs-cake/master/ac-cake.el")
;; (install-elisp "https://raw.github.com/k1LoW/emacs-cake/master/cake-inflector.el")
;; (install-elisp "https://raw.github.com/k1LoW/emacs-cake/master/cake.el")
(when (require 'cake nil t)
  (global-cake t))


;;; css-mode
;; (install-elisp "http://www.garshol.priv.no/download/software/css-mode/css-mode.el")
(when (require 'css-mode nil t)
  (setq cssm-indent-function #'cssm-c-style-indenter)
  (setq cssm-indent-level 2)
  (setq-default indent-tabs-mode nil)
  (setq cssm-newline-before-closing-bracket t))


;;; scratch バッファを次回起動時に復元。ログも記録する。
;; (install-elisp "https://raw.github.com/mori-dev/scratch-log/master/scratch-log.el")
(when (require 'scratch-log.el nil t)
  (setq sl-scratch-log-file "~/.emacs.d/.scratch-log")
  (setq sl-prev-scratch-string-file "~/.emacs.d/.scratch")
  ;; nil なら emacs 起動時に，最後に終了したときの スクラッチバッファの内容を復元しない。初期値は t です。
  (setq sl-restore-scratch-p t)
  ;; nil なら スクラッチバッファを削除できるままにする。初期値は t です。
  (setq sl-prohibit-kill-scratch-buffer-p t))


;;; 最近閉じたバッファを復元
;; http://d.hatena.ne.jp/kitokitoki/20100608/p2
(defvar my-killed-file-name-list nil)

(defun my-push-killed-file-name-list ()
  (when (buffer-file-name)
    (push (expand-file-name (buffer-file-name)) my-killed-file-name-list)))

(defun my-pop-killed-file-name-list ()
  (interactive)
  (unless (null my-killed-file-name-list)
    (find-file (pop my-killed-file-name-list))))

(add-hook 'kill-buffer-hook 'my-push-killed-file-name-list)
(define-key global-map (kbd "s-z") 'my-pop-killed-file-name-list)
